import attr
import cattr
import json
from typing import Generic, Optional, Type, TypeVar

T = TypeVar("T")


@attr.dataclass
class DapodikResult(Generic[T]):
    results: int
    id: str
    start: int
    limit: int
    rows: Optional[T] = None

    @classmethod
    def from_str(cls, data: str, cl: Type[T]):
        data_dict: dict = json.loads(data)
        if "rows" in data_dict and data_dict["rows"]:
            rows = cattr.structure(data_dict["rows"], cl)
            return cls(
                results=data_dict["results"],
                id=data_dict["id"],
                start=data_dict["start"],
                limit=data_dict["limit"],
                rows=rows,
            )
        return cls(
            results=0,
            id="",
            start=0,
            limit=0,
            rows=None,
        )



from cachetools import cachedmethod
from cachetools.keys import hashkey
from functools import partial, wraps
from operator import attrgetter
from typing import Callable, List, TypeVar

T = TypeVar("T")


def cached(key: str, cache_prop: str = "cache"):
    def wrapper(func):
        return wraps(func)(
            cachedmethod(attrgetter(cache_prop), key=partial(hashkey, key))(func)
        )

    return wrapper


def clean_response(data: str) -> str:
    data = data.replace("'success'", '"success"')
    data = data.replace("'message' : '", '"message" : "')
    if "rows" in data:
        data = data.replace("', 'rows'", '", "rows"')
    else:
        data = data.replace("' }", '" }')
    return data


def make_query(*args, **kwargs) -> dict:
    query = dict()
    for key, val in dict(kwargs).items():
        if val is not None:
            query[key] = val
    return query


def find_in(obj: List[T], is_this: Callable[[T], bool]) -> T:
    for o in obj:
        if is_this(o):
            return o
    raise ValueError("No obj found")



from typing import List

from dapodik.base import BaseDapodik
from dapodik.utils.helper import cached

from . import Wilayah


class BaseCustomrest(BaseDapodik):
    @cached("kecamatan")
    def kecamatan(
        self, query: str, page: int = 1, start: int = 0, limit: int = 50
    ) -> List[Wilayah]:
        return self._get_rest(
            "kecamatan",
            List[Wilayah],
            page,
            start,
            limit,
            query=self._query(query=query),
            prefix="customrest/",
        )



import attr
import cattr
import json
from typing import Generic, Optional, Type, TypeVar

T = TypeVar("T")


@attr.dataclass
class DapodikResponse(Generic[T]):
    success: bool
    message: str
    rows: Optional[T] = None

    @classmethod
    def from_str(cls, data: str, cl: Type[T]):
        data = data.replace("'success'", '"success"')
        data = data.replace("'message' : '", '"message" : "')
        data = data.replace("', 'rows'", '", "rows"')
        data_dict: dict = json.loads(data)
        if "rows" in data_dict and data_dict["rows"]:
            rows = cattr.structure(data_dict["rows"], cl)
            return cls(
                success=data_dict["success"], message=data_dict["message"], rows=rows
            )
        return cls(
            success=data_dict["success"], message=data_dict["message"], rows=None
        )



import attr
from datetime import datetime
from typing import Optional


@attr.dataclass(frozen=True, slots=True)
class JenisLk:
    id_jenis_lk: str
    nm_jenis_lk: str
    ket_jenis_lk: Optional[str]
    create_date: datetime
    last_update: datetime
    expired_date: Optional[datetime]
    last_sync: datetime

    def __str__(self):
        return self.nm_jenis_lk



import cattr
from datetime import datetime, date
from typing import Union
from uuid import UUID


def str_to_datetime(
    data: Union[str, datetime], format: str = "%Y-%m-%d %H:%M:%S"
) -> datetime:
    if isinstance(data, datetime):
        return data
    elif type(data) == str:
        if "." in data:
            return datetime.strptime(data, format + ".%f")
        return datetime.strptime(data, format)
    else:
        return datetime.now()


def str_to_date(data: Union[str, date], format: str = "%Y-%m-%d") -> date:
    if isinstance(data, date):
        return data
    elif type(data) == str:
        return datetime.strptime(data, format).date()
    else:
        return datetime.now().date()


def datetime_to_str(data: datetime) -> str:
    if isinstance(data, str):
        return data
    return data.strftime("%Y-%m-%d %H:%M:%S")


def date_to_str(data: date) -> str:
    if isinstance(data, str):
        return data
    return data.strftime("%Y-%m-%d")


def register_hooks():
    # dict -> attr
    cattr.register_structure_hook(date, lambda d, t: str_to_date(d))
    cattr.register_structure_hook(datetime, lambda d, t: str_to_datetime(d))
    cattr.register_structure_hook(UUID, lambda d, t: UUID(d))
    # attr -> dict
    cattr.register_unstructure_hook(date, date_to_str)
    cattr.register_unstructure_hook(datetime, datetime_to_str)
    cattr.register_unstructure_hook(UUID, str)



import attr
from datetime import date
from typing import List, Optional, TYPE_CHECKING
from uuid import UUID

if TYPE_CHECKING:
    from dapodik import Dapodik
from . import PesertaDidik


@attr.dataclass(kw_only=True)
class CreatePesertaDidik:
    """Data untuk membuat peserta didik"""

    # Data Pribadi
    nama: str
    jenis_kelamin: str
    nisn: str
    nik: str
    no_kk: str
    tempat_lahir: str
    tanggal_lahir: date
    reg_akta_lahir: str = ""
    kebutuhan_khusus_id: int = 0
    agama_id: int
    alamat_jalan: str
    rt: int = 0
    rw: int = 0
    nama_dusun: str
    kode_wilayah: str
    kode_wilayah_str: str
    desa_kelurahan: str
    kode_pos: int
    lintang: int
    bujur: int
    jenis_tinggal_id: int
    alat_transportasi_id: int
    anak_keberapa: int = 1
    penerima_kps: int = 0
    penerima_kip: int = 0
    layak_pip: int = 0
    id_layak_pip: Optional[int] = None
    sekolah_id: UUID
    kewarganegaraan: str = "ID"
    no_kks: str = ""
    no_kps: str = ""
    no_kip: str = ""
    nm_kip: int = 0
    pekerjaan_id: int = 0
    pekerjaan_id_str: str = ""
    # Data Ayah Kandung
    nama_ayah: str
    nik_ayah: str
    tahun_lahir_ayah: int
    jenjang_pendidikan_ayah: int
    pekerjaan_id_ayah: int
    penghasilan_id_ayah: int
    kebutuhan_khusus_id_ayah: int = 0
    # Data Ibu Kandung
    nama_ibu_kandung: str
    nik_ibu: str
    tahun_lahir_ibu: int
    jenjang_pendidikan_ibu: int
    pekerjaan_id_ibu: int
    penghasilan_id_ibu: int
    kebutuhan_khusus_id_ibu: int = 0
    # Data Wali
    nama_wali: str = ""
    nik_wali: str = ""
    tahun_lahir_wali: int
    jenjang_pendidikan_wali: int = 0
    pekerjaan_id_wali: int = 0
    penghasilan_id_wali: int = 0
    # Kontak
    nomor_telepon_rumah: str = ""
    nomor_telepon_seluler: str = ""
    email: str = ""
    # Bank PIP
    id_bank: str = ""
    rekening_bank: str = ""
    nama_kcp: str = ""
    rekening_atas_nama: str = ""
    # Etc
    peserta_didik_id: str = "Admin.model.PesertaDidik-1"
    kewarganegaraan_str: str = ""
    agama_id_str: str = ""
    status_data: int = 0
    pdb_id: str = ""
    kebutuhan_khusus_id_str: str = ""
    jenis_tinggal_id_str: str = ""
    alat_transportasi_id_str: str = ""
    id_layak_pip_str: str = ""
    id_bank_str: str = ""
    jenjang_pendidikan_ayah_str: str = ""
    jenjang_pendidikan_ibu_str: str = ""
    jenjang_pendidikan_wali_str: str = ""
    pekerjaan_id_ayah_str: str = ""
    pekerjaan_id_ibu_str: str = ""
    pekerjaan_id_wali_str: str = ""
    penghasilan_id_ayah_str: str = ""
    penghasilan_id_ibu_str: str = ""
    penghasilan_id_wali_str: str = ""
    kebutuhan_khusus_id_ayah_str: str = ""
    kebutuhan_khusus_id_ibu_str: str = ""
    kebutuhan_khusus_id_selector: List[int] = attr.ib(factory=list)
    kebutuhan_khusus_id_selector_ayah: List[int] = attr.ib(factory=list)
    kebutuhan_khusus_id_selector_ibu: List[int] = attr.ib(factory=list)
    vld_count: int = 0

    def save(
        self,
        dapodik: "Dapodik",
        pd_module: str = "pdterdaftar",
        limit: int = 25,
        ascending: str = "nama",
    ) -> PesertaDidik:
        query = {
            "sekolah_id": self.sekolah_id,
            "pd_module": pd_module,
            "limit": limit,
            "ascending": ascending,
        }
        return dapodik._post_rest(
            path="PesertaDidik", cl=PesertaDidik, data=self, query=query
        )



import attr
from datetime import datetime
from typing import Optional


@attr.dataclass(frozen=True, slots=True)
class Wilayah:
    kode_wilayah: str
    nama: str
    id_level_wilayah: int
    mst_kode_wilayah: str
    negara_id: str
    asal_wilayah: str
    kode_bps: str
    kode_dagri: str
    kode_keu: str
    id_prov: Optional[int]
    id_kabkota: Optional[int]
    id_kec: Optional[int]
    a_desa: int
    a_kelurahan: int
    a_35: int
    a_urban: int
    kategori_desa_id: Optional[int]
    create_date: datetime
    last_update: datetime
    expired_date: Optional[datetime]
    last_sync: datetime
    nama_kabupaten: str



import attr
from datetime import datetime, date
from typing import Optional, TYPE_CHECKING
from uuid import UUID

if TYPE_CHECKING:
    from dapodik import Dapodik

from . import PesertaDidikLongitudinal
from . import RegistrasiPesertaDidik


@attr.dataclass
class PesertaDidik:
    peserta_didik_id: UUID
    nama: str
    jenis_kelamin: str
    nisn: str
    nik: str
    no_kk: str
    tempat_lahir: str
    tanggal_lahir: date
    agama_id: int
    kebutuhan_khusus_id: int
    alamat_jalan: str
    rt: int
    rw: int
    nama_dusun: str
    desa_kelurahan: str
    kode_wilayah: str
    kode_pos: Optional[int]
    lintang: str
    bujur: str
    jenis_tinggal_id: int
    alat_transportasi_id: int
    nik_ayah: str
    nik_ibu: str
    anak_keberapa: int
    nik_wali: Optional[str]
    nomor_telepon_rumah: Optional[str]
    nomor_telepon_seluler: Optional[str]
    email: Optional[str]
    penerima_kps: int
    no_kps: Optional[str]
    layak_pip: int
    penerima_kip: int
    no_kip: Optional[str]
    nm_kip: Optional[str]
    no_kks: Optional[str]
    reg_akta_lahir: str
    id_layak_pip: Optional[int]
    id_bank: Optional[int]
    rekening_bank: Optional[str]
    nama_kcp: Optional[str]
    rekening_atas_nama: Optional[str]
    status_data: Optional[int]
    nama_ayah: str
    tahun_lahir_ayah: int
    jenjang_pendidikan_ayah: int
    pekerjaan_id_ayah: int
    penghasilan_id_ayah: int
    kebutuhan_khusus_id_ayah: int
    nama_ibu_kandung: str
    tahun_lahir_ibu: int
    jenjang_pendidikan_ibu: int
    penghasilan_id_ibu: int
    pekerjaan_id_ibu: int
    kebutuhan_khusus_id_ibu: int
    nama_wali: Optional[str]
    tahun_lahir_wali: Optional[str]
    jenjang_pendidikan_wali: int
    pekerjaan_id_wali: int
    penghasilan_id_wali: int
    kewarganegaraan: str
    yatim_piatu: Optional[bool] = None
    nama_sorter: Optional[str] = None
    rombel_saat_ini: Optional[str] = None
    rombel: Optional[str] = None
    # Ujian
    no_skhun: Optional[str] = None
    no_peserta_ujian: Optional[str] = None
    no_seri_ijazah: Optional[str] = None
    # Meta
    keterangan: Optional[str] = None
    # Registrasi
    nipd: Optional[str] = None
    nomor_induk_pd: Optional[int] = None
    tanggal_masuk_sekolah: Optional[date] = None
    sekolah_asal: Optional[str] = None
    a_pernah_paud: Optional[int] = None
    a_pernah_tk: Optional[int] = None
    id_hobby: Optional[int] = None
    id_cita: Optional[int] = None
    jenis_keluar_id: Optional[int] = None
    tanggal_keluar: Optional[date] = None
    konfirmasi_mutasi: Optional[int] = None
    ket_keluar: Optional[str] = None
    anggota_rombel_id: Optional[UUID] = None
    tingkat_pendidikan_id: Optional[int] = None
    jenis_pendaftaran_id: Optional[int] = None
    jenis_pendaftaran_id_str: str = ""
    jurusan_sp_id: Optional[str] = None
    sekolah_id: Optional[UUID] = None
    registrasi_id: Optional[str] = None
    vld_count: Optional[int] = None
    # Str
    id_bank_str: Optional[str] = None
    kewarganegaraan_str: Optional[str] = None
    # Metadata
    create_date: Optional[datetime] = None
    last_update: Optional[datetime] = None
    soft_delete: Optional[int] = None
    last_sync: Optional[datetime] = None
    updater_id: Optional[UUID] = None
    _dapodik: Optional["Dapodik"] = None

    def __str__(self):
        return self.nama

    @property
    def dapodik(self) -> "Dapodik":
        return self._dapodik  # type: ignore

    def registrasi(self, registrasi: RegistrasiPesertaDidik) -> RegistrasiPesertaDidik:
        registrasi.peserta_didik_id = self.peserta_didik_id
        if self.sekolah_id:
            registrasi.sekolah_id = self.sekolah_id
        registrasi = self.dapodik._post_rest(
            path="RegistrasiPesertaDidik",
            cl=RegistrasiPesertaDidik,
            data=registrasi,
        )
        self.nipd = registrasi.nipd
        self.registrasi_id = registrasi.registrasi_id
        self.tanggal_masuk_sekolah = registrasi.tanggal_masuk_sekolah
        self.sekolah_asal = registrasi.sekolah_asal
        self.a_pernah_paud = registrasi.a_pernah_paud
        self.a_pernah_tk = registrasi.a_pernah_tk
        self.id_hobby = registrasi.id_hobby
        self.id_cita = registrasi.id_cita
        if registrasi.jenis_keluar_id:
            self.jenis_keluar_id = int(registrasi.jenis_keluar_id)
        self.tanggal_keluar = registrasi.tanggal_keluar
        return registrasi

    def create_longitudinal(
        self, longitudinal: PesertaDidikLongitudinal.Create
    ) -> PesertaDidikLongitudinal:
        longitudinal.peserta_didik_id = self.peserta_didik_id
        return self.dapodik._post_rest(
            path="PesertaDidikLongitudinal",
            cl=PesertaDidikLongitudinal,
            data=longitudinal,
        )



import attr
from datetime import datetime
from typing import Optional


@attr.dataclass(frozen=True, slots=True)
class JenisPtk:
    jenis_ptk_id: int
    jenis_ptk: str
    guru_kelas: str
    guru_matpel: str
    guru_bk: str
    guru_inklusi: str
    guru_pengganti: str
    pengawas_satdik: str
    pengawas_plb: str
    pengawas_matpel: str
    pengawas_bidang: str
    tas: str
    tendik_lainnya: str
    formal: str
    create_date: datetime
    last_update: datetime
    expired_date: Optional[datetime]
    last_sync: datetime

    def __str__(self):
        return self.jenis_ptk



import attr
import cattr
import json
import logging
from cachetools import LRUCache
from requests import Response, Session
from typing import Any, Callable, MutableMapping, Optional, List, Type, TypeVar, Union

from dapodik.exception import DapodikResponseError, ServerTidakMerespon
from dapodik.utils.helper import clean_response, find_in, make_query
from dapodik.utils.parser import register_hooks

T = TypeVar("T")


class BaseDapodik(object):
    def __init__(self, base_url: str = "http://localhost:5774/"):
        self._logger = logging.getLogger("Dapodik")
        self._session = Session()
        self._base_url = base_url
        self._register_hooks()
        self._cache: MutableMapping = LRUCache(128)

    @property
    def cache(self) -> MutableMapping:
        return self._cache

    @property
    def logger(self) -> logging.Logger:
        return self._logger

    @property
    def session(self) -> Session:
        return self._session

    @property
    def base_url(self) -> str:
        return self._base_url

    def _url(self, path: str) -> str:
        if path.startswith(self.base_url):
            return path
        return self.base_url + path.lstrip("/")

    def _rest_url(self, name: str) -> str:
        return self._url("rest/" + name.lstrip("/"))

    def _post(
        self,
        url: str,
        data: dict = None,
        json: dict = None,
        params: dict = None,
        headers: dict = None,
        **kwargs: Any,
    ) -> Response:
        return self.session.post(
            self._url(url),
            data=data,
            json=json,
            params=params,
            headers=headers,
            **kwargs,
        )

    def _get(
        self,
        url: str,
        params: dict = None,
        **kwargs: Any,
    ) -> Response:
        return self.session.get(
            self._url(url),
            params=params,
            **kwargs,
        )

    def _get_rows(
        self,
        path: str,
        cl: Type[T],
        query: Optional[dict] = None,
        key: Callable[[Any], Any] = lambda x: x["rows"],
        **kwargs: Any,
    ) -> T:
        res = self._get(
            url=path,
            params=query,
            **kwargs,
        )
        if not res.ok:
            raise ServerTidakMerespon("Server tidak merespon")
        data: dict = json.loads(res.text)
        obj: Any = key(data) if callable(key) else data
        result = cattr.structure(obj, cl)
        if isinstance(result, list):
            for res in result:
                if not hasattr(res, "_dapodik"):
                    break
                setattr(res, "_dapodik", self)
        elif hasattr(result, "_dapodik"):
            setattr(res, "_dapodik", self)
        return result

    def _get_rest(
        self,
        path: str,
        cl: Type[T],
        page: int = 1,
        start: int = 9,
        limit: Union[int, str] = 50,
        query: Optional[dict] = None,
        prefix: str = "rest/",
        key: Callable[[Any], Any] = lambda x: x["rows"],
    ) -> T:
        params = {
            "page": page,
            "start": start,
            "limit": limit,
        }
        if query:
            params.update(query)
        return self._get_rows(prefix + path.lstrip("/"), cl=cl, query=params, key=key)

    def _post_rows(
        self,
        path: str,
        cl: Type[T],
        data: Optional[dict] = None,
        query: Optional[dict] = None,
        key: Callable[[Any], Any] = lambda x: x["rows"],
        **kwargs: Any,
    ) -> T:
        res = self._post(url=path, json=data, **kwargs)
        if not res.ok:
            raise ServerTidakMerespon("Server tidak merespon")
        raw_data: str = self._clean_response(res.text)
        res_data: dict = json.loads(raw_data)
        self.logger.debug(f"Response POST : {raw_data}")
        if "success" not in res_data:
            raise DapodikResponseError(f"No success message found in data `{raw_data}`")
        elif not res_data["success"]:
            if "message" in res_data:
                raise DapodikResponseError(res_data["message"])
            raise DapodikResponseError(f"Request failed with response :`{raw_data}`")
        obj: Any = key(res_data) if callable(key) else res_data
        result = cattr.structure(obj, cl)
        if isinstance(result, list):
            for res in result:
                if not hasattr(res, "_dapodik"):
                    break
                setattr(res, "_dapodik", self)
        elif hasattr(result, "_dapodik"):
            setattr(res, "_dapodik", self)
        return result

    def _post_rest(
        self,
        path: str,
        cl: Type[T],
        data: Any = None,
        query: dict = None,
        prefix: str = "rest/",
        key: Callable[[Any], Any] = lambda x: x["rows"],
    ):
        if data and attr.has(type(data)):
            data = cattr.unstructure(data)
        return self._post_rows(
            prefix + path.lstrip("/"), cl=cl, data=data, query=query, key=key
        )

    _clean_response = staticmethod(clean_response)
    _find = staticmethod(find_in)
    _register_hooks = staticmethod(register_hooks)
    _query = staticmethod(make_query)



class DapodikException(BaseException):
    pass


class AuthException(DapodikException):
    pass


class PasswordSalah(AuthException):
    pass


class PenggunaTidakTerdaftar(AuthException):
    pass


class ServerTidakMerespon(DapodikException):
    pass


class DapodikResponseError(DapodikException):
    pass



import attr
from datetime import datetime
from typing import Optional


@attr.dataclass(frozen=True, slots=True)
class JenisSarana:
    jenis_sarana_id: int
    nama: str
    kelompok: Optional[str]
    perlu_penempatan: int
    keterangan: str
    a_alat: int
    a_angkutan: int
    spm_qty_min_per_siswa: str
    spm_qty_min_per_sekolah: int
    create_date: datetime
    last_update: datetime
    expired_date: Optional[datetime]
    last_sync: datetime
